use std::{
	collections::{HashMap, VecDeque},
	io::{self, Write},
};

pub fn run(program: Vec<u8>) {
	let mut tape: Vec<u8> = vec![0];
	let mut cell_index: usize = 0;

	let mut ip: usize = 0;
	let mut icount: usize = 0;

	let mut table: HashMap<usize, usize> = HashMap::new();
	let mut stack: VecDeque<usize> = VecDeque::new();
	for (ip, instruction) in program.iter().enumerate() {
		if *instruction == b'[' {
			stack.push_back(ip);
		}
		else if *instruction == b']' {
			let begin_index = stack.pop_back().unwrap();
			table.insert(begin_index, ip);
			table.insert(ip, begin_index);
		}
	}

	while ip < program.len() {
		let Some(instruction) = program.get(ip)
		else {
			break;
		};

		match instruction {
			b'+' => {
				tape[cell_index] = tape[cell_index].wrapping_add(1);
				icount += 1;
			}
			b'-' => {
				tape[cell_index] = tape[cell_index].wrapping_sub(1);
				icount += 1;
			}
			b'>' => {
				cell_index = cell_index.wrapping_add(1);
				if tape.len() <= cell_index {
					tape.push(0);
				}
				icount += 1;
			}
			b'<' => {
				cell_index = cell_index.wrapping_sub(1);
				icount += 1;
			}
			b'.' => {
				print!("{}", tape[cell_index] as char);
				icount += 1;
			}
			b',' => {
				tape[cell_index] = input().remove(0);
				icount += 1;
			}
			b'[' => {
				if tape[cell_index] == 0 {
					ip = *table.get(&ip).unwrap();
				}
				icount += 1;
			}
			b']' => {
				if tape[cell_index] != 0 {
					ip = *table.get(&ip).unwrap();
				}
				icount += 1;
			}
			_ => {}
		}

		ip += 1;
	}

	println!();
	println!("Instructions ran: {icount}");
	// println!("{tape:?}");
}

fn input() -> Vec<u8> {
	let mut input = String::new();

	print!("Input: ");
	let _ = io::stdout().flush();
	let _ = io::stdin().read_line(&mut input);

	input.as_bytes().to_vec()
}
